#!BPY
"""
Name: 'Hyperboloid'
Blender: 244
Group: 'AddMesh'
"""
import BPyAddMesh
import Blender
from math import sqrt, pi, sin, cos

def add_hyperboloid(CONTINUITY, RESOLUTION, RINGS):
	v = []
	f = []
	
	#Creating the vertices
	z_step = 100.0/(RINGS-1)
	for k in range(-1,2,2):
		for i in range(0,RINGS):
			i*=k
			z = i*z_step
			radius = sqrt(z**2-CONTINUITY)
			for j in range(0,RESOLUTION):
				angle = float(j)/RESOLUTION*2*pi
				x = cos(angle)*radius
				y = sin(angle)*radius
				v.append([x/100,y/100,z/100])
	v = v[RESOLUTION:len(v)]	
	
	#Creating the faces of the lower half
	for j in range(0,RINGS-1):
		for i in range(0,RESOLUTION):
			if(i==RESOLUTION-1):
				c = RESOLUTION
			else:
				c = 0
			i+=j*RESOLUTION
			f.append([i,i+RESOLUTION,i+RESOLUTION+1-c,i+1-c])
	f = f[0:-RESOLUTION]
	#Creating the faces of the lower half connecting with the upper half
	for i in range(0,RESOLUTION):
		if(i==RESOLUTION-1):
			c = RESOLUTION
		else:
			c = 0
		i+=j*RESOLUTION
		f.append([i+RESOLUTION*(-RINGS+2),i+1-c+RESOLUTION*(-RINGS+2),i+RESOLUTION+1-c,i+RESOLUTION])
	#Creating the faces of the upper half
	for j in range(RINGS-1,2*RINGS-2):
		for i in range(0,RESOLUTION):
			if(i==RESOLUTION-1):
				c = RESOLUTION
			else:
				c = 0
			i+=j*RESOLUTION
			f.append([i,i+1-c,i+RESOLUTION+1-c,i+RESOLUTION])
	
	return v, f

def main():
	CONTINUITY = Blender.Draw.Create(3.00)
	RESOLUTION = Blender.Draw.Create(32)
	RINGS = Blender.Draw.Create(5)

	if not Blender.Draw.PupBlock('Add Hyperboloid', [\
	('Centre:', CONTINUITY,  0.00, 10.00, 'Influences the radius of the center'),\
	('Resolution:', RESOLUTION, 3, 500, 'Number of vertices per ring'),\
	('Rings:', RINGS, 3, 500, 'Number of rings in each half of the hyperboloid'),\
	]):
		return
	
	v, f = add_hyperboloid(-1000*CONTINUITY.val, RESOLUTION.val, RINGS.val)
	
	BPyAddMesh.add_mesh_simple('Hyperboloid', v, [], f)

main()