#!BPY

"""
Name: 'Bolt'
Blender: 244
Group: 'AddMesh'
"""

__author__= ['Luis Sergio S. Moura Jr.']
__url__ = ("liquidblue.com.br")
__version__= '0.1'
__bpydoc__= '''\

Bolt Mesh

0.1 - 2007-07-27 by Luis Sergio<br />
- initial version

'''

# *** BEGIN LICENSE BLOCK ***
# Creative Commons 3.0 by-sa
#
# Full license: http://creativecommons.org/licenses/by-sa/3.0/
# Legal code: http://creativecommons.org/licenses/by-sa/3.0/legalcode
#
# You are free:
# * to Share  to copy, distribute and transmit the work
# * to Remix  to adapt the work
#
# Under the following conditions:
# * Attribution. You must attribute the work in the manner specified by the author or licensor (but not in any way that suggests that they endorse you or your use of the work).
# * Share Alike. If you alter, transform, or build upon this work, you may distribute the resulting work only under the same, similar or a compatible license.
#
# * For any reuse or distribution, you must make clear to others the license terms of this work. The best way to do this is with a link to this web page.
# * Any of the above conditions can be waived if you get permission from the copyright holder.
# * Nothing in this license impairs or restricts the author's moral rights.
# *** END LICENSE BLOCK ***


import BPyAddMesh
import Blender
import math

# Returns a 3d vector given angle, radius and z (optional)
def point(ANGLE, RADIUS, DZ = 0):
	vert = [RADIUS * math.sin(ANGLE), RADIUS * math.cos(ANGLE), DZ]
	return vert


# actually creates the vertices and the faces for the bolt.
def create_bolt(RADIUS, SUBDIVISIONS, STEP, LOOPS, IDENT):
	verts = []
	faces = []

	# ========= VERTS =========
	# cap
	for i in range(SUBDIVISIONS):
		# we start one loop BEFORE the other loops, so the 0 is one step before the others.
		deg = math.pi * 2 * (i - 1) / SUBDIVISIONS
		if (i == 0):
			verts.append(point(deg, RADIUS - IDENT))
		else:
			verts.append(point(deg, RADIUS))

	# 1st loop
	for i in range(SUBDIVISIONS):	
		deg = math.pi * 2 * i / SUBDIVISIONS
		space = (i + 1) * STEP / (SUBDIVISIONS * 2)
		verts.append(point(deg, RADIUS - IDENT, space))
		verts.append(point(deg, RADIUS, space * 2))

	# other loops
	disp = 0
	space = STEP / 2
	for j in range(1, LOOPS - 1):
		for i in range(SUBDIVISIONS):
			deg = math.pi * 2 * i / SUBDIVISIONS
			disp = disp + STEP / SUBDIVISIONS
			verts.append(point(deg, RADIUS - IDENT, disp + space))
			verts.append(point(deg, RADIUS, disp + space * 2))

	# last one
	lastdisp = disp + space * 2
	disp = disp + space
	for i in range(SUBDIVISIONS - 1):
		disp = disp + (STEP / SUBDIVISIONS) / 2
		deg = math.pi * 2 * i / SUBDIVISIONS
		verts.append(point(deg, RADIUS - IDENT, disp))
		verts.append(point(deg, RADIUS, lastdisp))

	# last cap
#	for i in range(SUBDIVISIONS - 1):
#		deg = math.pi * 2 * i / SUBDIVISIONS
	

	# ========= FACES =========
	# cap / 1st loop
	# 	face count: 16 (SUBDIVISIONS * 2)
	# 	vertice count: 22 (SUBDIVISIONS * 3 - 2)
	faces.append([1, 0, SUBDIVISIONS])	
	faces.append([0, SUBDIVISIONS + 1, SUBDIVISIONS])

	for i in range(1, SUBDIVISIONS):
		i2 = (i - 1) * 2
		if (i == SUBDIVISIONS - 1):
			faces.append([0, i, SUBDIVISIONS + i2, SUBDIVISIONS + i2+2])
		else:
			faces.append([i+1, i, SUBDIVISIONS + i2, SUBDIVISIONS + i2+2])
		faces.append([SUBDIVISIONS + i2 + 2, SUBDIVISIONS + i2, SUBDIVISIONS + i2 + 1, SUBDIVISIONS + i2 + 3])

	# 2nd to n - 1
	lastcorner = 0
	nextcorner = 9
	othercorner = 0
	vcount = SUBDIVISIONS * 3 - 2
	for j in range(1, LOOPS - 1):
		for i in range(SUBDIVISIONS):
			othercorner = nextcorner + vcount - SUBDIVISIONS - 1
			faces.append([nextcorner, lastcorner, othercorner, othercorner+2])
			faces.append([othercorner+2, othercorner, othercorner+1, othercorner+3])
			lastcorner = nextcorner
			nextcorner = nextcorner + 2


	# last one!
	eol = othercorner + 3 # stands for end of line
	for i in range(1, SUBDIVISIONS):
		othercorner = nextcorner + vcount - SUBDIVISIONS - 1
		faces.append([nextcorner, lastcorner, othercorner, othercorner + 2])
		faces.append([othercorner+2, othercorner, othercorner+1, othercorner+3])

		if (i == SUBDIVISIONS - 1):
			faces.append([eol, nextcorner, othercorner+2])
			faces.append([eol, othercorner+2, othercorner+3])
		lastcorner = nextcorner
		nextcorner = nextcorner + 2



	return verts, faces


# main function (window handle and input variables)
def main():
	verts = []
	faces = []

	boltRadiusInput = Blender.Draw.Create(1.0)
	boltDivInput = Blender.Draw.Create(8)
	boltStepInput = Blender.Draw.Create(.2)
	boltLoopsInput = Blender.Draw.Create(5)
	boltIdentInput = Blender.Draw.Create(.1)
	
	block = []
	block.append(("Radius:", boltRadiusInput, 0.01, 100, "the inner radius"))
	block.append(("Divisions:", boltDivInput, 4, 100, "the outer radius"))
	block.append(("Step:", boltStepInput, .01, 100, "step length"))
	block.append(("Loops:", boltLoopsInput, 3, 100, "the height of the pipe"))
	block.append(("Ident:", boltIdentInput, .01, 100, "number of height divisions"))
	
	if not Blender.Draw.PupBlock("Create bolt", block):
		return

#	Generates the mesh
	verts, faces = create_bolt(boltRadiusInput.val,boltDivInput.val,boltStepInput.val,boltLoopsInput.val,boltIdentInput.val)

	#print '== VERTS =='
	#print verts
	#print '== FACES =='
	#print faces

#	Adds the mesh to the scene
	BPyAddMesh.add_mesh_simple('Bolt', verts, [], faces)

# call our main function	
main()