#!BPY

"""
Name: 'Bake Second Life Sculpties'
Blender: 244
Group: 'Render'
Tooltip: 'Bake Sculptie Maps on Active objects'
"""

__author__ = ["Domino Marama"]
__url__ = ("http://dominodesigns.info")
__version__ = "0.07"
__bpydoc__ = """\

Bake Sculptie Map

This script requires a square planar mapping. It bakes the local vertex
positions to the mesh's uv image.
"""
#Changes

#0.07 Domino Marama 2008-02-10
#- Bake routine rewritten
#0.06 Domino Marama 2007-12-23
#- fixed sculptie type
#0.05 Domino Marama 2007-08-29
#- code cleanup
#- added prim properties
#- changed console print out to LSL
#0.04 Domino Marama 2007-08-26
#- Fixed fill routine and added options for faces & fill
#0.03 Domino Marama 2007-08-19
#- Various fixes to triangle fill routines
#0.02 Domino Marama 2007-08-18
#- Converted to triangle fill routines
#0.01 Domino Marama 2007-08-10
#- Initial Version

# ***** BEGIN GPL LICENSE BLOCK *****
#
# Script copyright (C) Domino Designs Limited
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software Foundation,
# Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
#
# ***** END GPL LICENCE BLOCK *****
# --------------------------------------------------------------------------

#***********************************************
# Import modules
#***********************************************

import Blender

class colvert:
	def __init__(self, x, y, r, g, b):
	        self.x = x
		self.y = y
		self.r = r
		self.g = g
		self.b = b
	def __cmp__(self, other):
		if self.x == other.x:
			return cmp(self.y, other.y)
		else:
			return cmp(self.x, other.x)

def drawTri( image, verts ):
	scanLines = [ verts[0] ]
	if verts[0].x != verts[1].x:
		scanLines.extend( [ verts[0] ])
		if verts[1].x == verts[2].x:
			scanLines.extend( [ verts[1], verts[2] ] )
		else:
			s = ( verts[1].x - verts[0].x ) / ( verts[2].x - verts[0].x )
			n = colvert(
				verts[0].x + (verts[2].x - verts[0].x ) * s,
				verts[0].y + (verts[2].y - verts[0].y ) * s,
				verts[0].r + (verts[2].r - verts[0].r ) * s,
				verts[0].g + (verts[2].g - verts[0].g ) * s,
				verts[0].b + (verts[2].b - verts[0].b ) * s
			)
			if verts[1].y < verts[2].y:
				scanLines.extend( [ verts[1], n, verts[1], n, verts[2], verts[2] ] )
			else:
				scanLines.extend( [ n, verts[1], n, verts[1], verts[2], verts[2] ] )
	else:
		scanLines.extend( [ verts[1], verts[2], verts[2] ] )
	while scanLines:
		s1 = scanLines[ 0 ]
		s2 = scanLines[ 1 ]
		e1 = scanLines[ 2 ]
		e2 = scanLines[ 3 ]
		scanLines = scanLines[4:]
		ry1 = e1.y - s1.y
		ry2 = e2.y - s2.y
		rr1 = e1.r - s1.r
		rr2 = e2.r - s2.r
		rg1 = e1.g - s1.g
		rg2 = e2.g - s2.g
		rb1 = e1.b - s1.b
		rb2 = e2.b - s2.b
		x = s1.x
		rx = e1.x - s1.x
		while x <= e1.x and x < image.size[0]:
			s = ( x - s1.x ) / rx
			drawVLine( image, int(x),
				int(s1.y + ry1 * s),
				int(s2.y + ry2 * s),
				s1.r + rr1 * s,
				s1.g + rg1 * s,
				s1.b + rb1 * s,
				s2.r + rr2 * s,
				s2.g + rg2 * s,
				s2.b + rb2 * s
			)
			x += 1.0
			
def drawLine( image, y, s, e, sr, sg, sb, er, eg, eb ):
	if s - e == 0:
		image.setPixelF( s, y, ( sr, sg, sb, 1.0 ) )
		return
	dr = ( er - sr ) / ( e - s )
	dg = ( eg - sg ) / ( e - s )
	db = ( eb - sb ) / ( e - s )
	for u in xrange( s, e + 1 ):
		image.setPixelF( u, y, ( sr, sg, sb, 1.0 ) )
		sr += dr
		sg += dg
		sb += db
		if sr < 0:
			sr = 0
		if sg < 0:
			sg = 0
		if sb < 0:
			sb = 0
		if sr > 1.0:
			sr = 1.0
		if sg > 1.0:
			sg = 1.0
		if sb > 1.0:
			sb = 1.0

def drawVLine( image, x, s, e, sr, sg, sb, er, eg, eb ):
	if s - e == 0:
		if s == image.size[1]:
			s -= 1
		image.setPixelF( x, s, ( sr, sg, sb, 1.0 ) )
		return
	dr = ( er - sr ) / ( e - s )
	dg = ( eg - sg ) / ( e - s )
	db = ( eb - sb ) / ( e - s )
	for v in xrange( s, e + 1 ):
		if v == image.size[1]:
			image.setPixelF( x, v - 1, ( sr, sg, sb, 1.0 ) )
		else:
			image.setPixelF( x, v, ( sr, sg, sb, 1.0 ) )
		sr += dr
		sg += dg
		sb += db
		if sr < 0:
			sr = 0
		if sg < 0:
			sg = 0
		if sb < 0:
			sb = 0
		if sr > 1.0:
			sr = 1.0
		if sg > 1.0:
			sg = 1.0
		if sb > 1.0:
			sb = 1.0
	
#***********************************************
# bake UV map from XYZ
#***********************************************

block = []
doFill = Blender.Draw.Create( True )
block.append (( "Fill Holes", doFill ))

if Blender.Draw.PupBlock( "Sculptie Bake Options", block ):
	print "--------------------------------"
	time1 = Blender.sys.time()  #for timing purposes
	scene = Blender.Scene.GetCurrent()
	editmode = Blender.Window.EditMode()
	if editmode: Blender.Window.EditMode(0)
	minx = None
	if doFill == 1:
		doFill = True
	else:
		doFill = False
	for ob in scene.objects.selected:
		if ob.type == 'Mesh':
			mesh = ob.getData( False, True )
			if minx == None:
				minx = mesh.verts[0].co.x
				maxx = minx
				miny = mesh.verts[0].co.y
				maxy = miny
				minz = mesh.verts[0].co.z
				maxz = minz
			for v in mesh.verts[1:-1]:
				if v.co.x < minx :
					minx = v.co.x
				elif v.co.x > maxx :
					maxx = v.co.x
				if v.co.y < miny :
					miny = v.co.y
				elif v.co.y > maxy :
					maxy = v.co.y
				if v.co.z < minz :
					minz = v.co.z
				elif v.co.z > maxz :
					maxz = v.co.z
	if minx == None:
		Blender.Draw.PupBlock( "Sculptie Bake Error", ["No objects selected"] )
	else:
		rangex = maxx - minx
		rangey = maxy - miny
		rangez = maxz - minz
		for ob in scene.objects.selected:
			if ob.type == 'Mesh':
				try:
					primtype = ob.getProperty( 'LL_PRIM_TYPE' ).getData()
					if primtype != 7:
						print "Skipping:", ob.name,"prim type is not a sculptie"
						continue
					else:
						sculptie_type = ob.getProperty( 'LL_SCULPT_TYPE' ).getData()
				except:
					sculptie_type = Blender.Draw.PupMenu( "Sculpt Type?%t|Sphere|Torus|Plane|Cylinder" )
					ob.addProperty( 'LL_PRIM_TYPE', 7 )
					ob.addProperty( 'LL_SCULPT_TYPE', sculptie_type )
				if sculptie_type < 1:
					print "Skipping:", ob.name, " - sculptie type is 'NONE'"
					continue
				mesh = ob.getData( False, True )
				sculptimage = mesh.faces[0].image
				scale = ob.getSize()
				if sculptimage == None:
					print "Warning:",mesh.name, "has no image"
					continue
				for u in xrange( sculptimage.size[0] ):
					for v in xrange( sculptimage.size[1] ):
						sculptimage.setPixelF(u, v, (0.0, 0.0, 0.0, 0.0))
				priminfo = ( sculptimage.name,
						["NONE", "SPHERE", "TORUS", "PLANE", "CYLINDER"][sculptie_type],
						rangex * scale[0],
						rangey * scale[1],
						rangez * scale[2]
						)
				print "llSetPrimitiveParams( [ PRIM_TYPE, PRIM_TYPE_SCULPT, \"%s\", PRIM_SCULPT_TYPE_%s, PRIM_SIZE, < %.5f, %.5f, %.5f > ] );"%priminfo

				for f in mesh.faces:
					for t in Blender.Geometry.PolyFill([ f.uv ]):
						nf = []
						for i in t:
							nf.append(
								colvert(
									f.uv[ i ][0] * sculptimage.size[0],
									f.uv[ i ][1] * sculptimage.size[1],
									( f.verts[ i ].co[0] - minx ) / rangex,
									( f.verts[ i ].co[1] - miny ) / rangey,
									( f.verts[ i ].co[2] - minz ) / rangez
								)
							)
						drawTri( sculptimage, sorted( nf ) )
				if doFill:
					def getFirstX( y ):
						for x in xrange( sculptimage.size[0] ):
							c = sculptimage.getPixelF( x, y )
							if c[3] != 0:
								if x > 0: doFill = True
								return c
						return None
					def getFirstY( x ):
						for y in xrange( sculptimage.size[1] ):
							c = sculptimage.getPixelF( x, y )
							if c[3] != 0:
								if y > 0: doFill = True
								return c
						return None
					def fillX():
						skipx = doFill = False
						for v in xrange( sculptimage.size[1] ):
							c = getFirstX( v )
							if not c : 
								skipx= True
								continue
							sr = c[0]
							sg = c[1]
							sb = c[2]
							s = 0
							for u in xrange( 1, sculptimage.size[0] ):
								nc = sculptimage.getPixelF( u, v )
								if nc[3] == 0:
									if not doFill:
										doFill = True
								else:
									if doFill:
										doFill = False
										drawLine( sculptimage, v, s, u, sr, sg, sb, nc[0], nc[1], nc[2] )
									s = u
									sr = nc[0]
									sg = nc[1]
									sb = nc[2]
							if doFill:
								doFill = False
								drawLine( sculptimage, v, s, u, sr, sg, sb, sr, sg, sb )
						return skipx
					def fillY():
						doFill = False
						for u in xrange( sculptimage.size[0] ):
							c = getFirstY( u )
							if not c :
								continue
							sr = c[0]
							sg = c[1]
							sb = c[2]
							s = 0
							for v in xrange( 1, sculptimage.size[1] ):
								nc = sculptimage.getPixelF( u, v )
								if nc[3] == 0:
									if not doFill:
										doFill = True
								else:
									if doFill:
										doFill = False
										drawVLine( sculptimage, u, s, v, sr, sg, sb, nc[0], nc[1], nc[2] )
									s = v
									sr = nc[0]
									sg = nc[1]
									sb = nc[2]
							if doFill:
								doFill = False
								drawVLine( sculptimage, u, s, v, sr, sg, sb, sr, sg, sb )
					skipx = fillX()
					fillY()
					if skipx: fillX()
		print "--------------------------------"
		print 'finished baking: in %.4f sec.' % ((Blender.sys.time()-time1))
		Blender.Redraw()
		if editmode: Blender.Window.EditMode(1)
